import type { PluginObj, PluginPass } from '@babel/core';
type State = PluginPass & {
    opts: Options;
};
/**
 * A callback function provided as a value to `Options.appendExtension` or to an
 * entry in `Options.replaceExtensions`.
 */
export type Callback<ReturnType> = (context: {
    /**
     * The import/export specifier being evaluated by Babel.
     */
    specifier: string;
    /**
     * An array of capturing groups returned by `String.prototype.match()` or an
     * empty array if the matcher string was not a regular expression.
     */
    capturingGroups: string[];
    /**
     * The absolute path of the file containing the specifier being evaluated by
     * Babel.
     */
    filepath: string;
}) => ReturnType;
/**
 * The options that can be passed to this plugin from babel.
 */
export type Options = {
    /**
     * This string will be appended to all relative import specifiers that do not
     * already have a recognized extension. Also accepts a callback function for
     * advanced use cases.
     *
     * @default undefined
     */
    appendExtension?: string | Callback<string | undefined>;
    /**
     * Members of this array will be considered a "recognized extension".
     *
     * @default defaultRecognizedExtensions
     */
    recognizedExtensions?: string[];
    /**
     * Map of specifiers to their replacements. Specifiers can be strings or
     * regular expressions (i.e. strings that start with ^ and/or end with $). If
     * a specifier is a regular expression, capturing group notation can be used
     * in the replacement. Replacements can either be a string or a callback
     * function that returns a string.
     *
     * @default {}
     */
    replaceExtensions?: Record<string, string | Callback<string>>;
    /**
     * If true, this plugin will generate no output.
     *
     * @default false
     */
    silent?: boolean;
    /**
     * If true, this plugin will generate more output than usual.
     *
     * @default false
     */
    verbose?: boolean;
};
/**
 * The default value of `Options.recognizedExtensions`.
 */
export declare const defaultRecognizedExtensions: readonly [".js", ".jsx", ".mjs", ".cjs", ".json"];
/**
 * A babel plugin that reliably rewrites import (and export) specifiers.
 */
export default function transformRewriteImports(): PluginObj<State>;
export {};
